#include <windows.h>
#include <algorithm>
#include "MQPlugin.h"
#include "MQ3DLib.h"
#include "voxel2.h"
#include <set>
#include <stack>
#include <float.h>
#include <assert.h>
#define ASSERT assert

struct RAYHITINFO{
	float fDist;	//C܂łɐi񂾋
	float fProd;	//ʖ@ƃC̓
};

struct RAYHITINFO_SORTER{
	bool operator()(const RAYHITINFO &a,const RAYHITINFO &b){
		return a.fDist<b.fDist;
	}
};

inline float fmax(float x,float y){return x>y? x:y;}
inline float fmin(float x,float y){return x<y? x:y;}
inline float pow2(float x){return x*x;}

template<typename T,typename U>
bool isIn(const T &collection,U value)
{
	return collection.end()!=std::find(collection.begin(),collection.end(),value);
}

template<typename T>
bool isIn(const T* p,size_t length,T value)
{
	for(size_t i=0;i<length;i++){
		if(p[i]==value)return true;
	}
	return false;
}


struct POLYGON_AABB_NODE{
	MQPoint vMin,vMax;
	DWORD dwFace;	//iq̖
	DWORD children[2];
};


bool Outer(float a1,float a2,float x1,float x2)
{
	//make sure a1<a2
	if(a1>a2){std::swap(a1,a2);}

	if(x1<a1&&x2<a1)return true;
	if(x1>a2&&x2>a2)return true;
	return false;
}

bool DXUtilCheckBoxHit(const MQPoint& vMax,const MQPoint& vMin,const MQPoint& vBegin,const MQPoint& vEnd)
{
	//---xNgZɂ铖蔻
	//̖ʂɑ΂Aꂼ̓_ƂɗLΊOĂ

	//X̃`FbN
	if(Outer(vMin.x,vMax.x,vBegin.x,vEnd.x))return false;
	//Ỹ`FbN
	if(Outer(vMin.y,vMax.y,vBegin.y,vEnd.y))return false;
	//Z̃`FbN
	if(Outer(vMin.z,vMax.z,vBegin.z,vEnd.z))return false;

	return true;
}

void getCombinedAABB(MQPoint &vMax,MQPoint &vMin,const MQPoint &amax,const MQPoint &amin,const MQPoint &bmax,const MQPoint &bmin)
{
	vMax.x=max(amax.x,bmax.x);
	vMax.y=max(amax.y,bmax.y);
	vMax.z=max(amax.z,bmax.z);

	vMin.x=min(amin.x,bmin.x);
	vMin.y=min(amin.y,bmin.y);
	vMin.z=min(amin.z,bmin.z);
}

//Ώە]֐
float combinationWeight(const MQPoint &amax,const MQPoint &amin,const MQPoint &bmax,const MQPoint &bmin)
{
	float dx=((amax.x+amin.x)-(bmax.x+bmin.x))*0.5f;
	float dy=((amax.y+amin.y)-(bmax.y+bmin.y))*0.5f;
	float dz=((amax.z+amin.z)-(bmax.z+bmin.z))*0.5f;

	return pow2(dx)+pow2(dy)+pow2(dz);
}


//{gAbvAABB\z
void DXUtilBuildPolygonAABBTree(
	std::vector<POLYGON_AABB_NODE> &rTree,
	DWORD idxSubLevels,	//̃x̊iqn܂ԍ
	const void* pVertices,
	size_t vertexStride,	//_z̈vf̑傫
	const DWORD numFaces,
	const DWORD *pIndexArray
	)
{
	const float EPSILON=5.0e-3f;

	DWORD numSubLevels=rTree.size()-idxSubLevels;
	DWORD dwOffset=rTree.size();

	if(numSubLevels==0){
		//ł჌x̊iq쐬
		rTree.resize(rTree.size()+numFaces);

		for(size_t i=0;i<numFaces;i++){
			DWORD i0=pIndexArray[i*3+0];
			DWORD i1=pIndexArray[i*3+1];
			DWORD i2=pIndexArray[i*3+2];

			const MQPoint &v0=*( (const MQPoint*) (((LPCBYTE)pVertices)+vertexStride*i0) );
			const MQPoint &v1=*( (const MQPoint*) (((LPCBYTE)pVertices)+vertexStride*i1) );
			const MQPoint &v2=*( (const MQPoint*) (((LPCBYTE)pVertices)+vertexStride*i2) );

			MQPoint vMin;
			vMin.x=min(min(v0.x,v1.x),v2.x)-EPSILON;
			vMin.y=min(min(v0.y,v1.y),v2.y)-EPSILON;
			vMin.z=min(min(v0.z,v1.z),v2.z)-EPSILON;

			MQPoint vMax;
			vMax.x=max(max(v0.x,v1.x),v2.x)+EPSILON;
			vMax.y=max(max(v0.y,v1.y),v2.y)+EPSILON;
			vMax.z=max(max(v0.z,v1.z),v2.z)+EPSILON;

			rTree[dwOffset+i].vMax=vMax;
			rTree[dwOffset+i].vMin=vMin;
			rTree[dwOffset+i].children[0]=-1;
			rTree[dwOffset+i].children[1]=-1;
			rTree[dwOffset+i].dwFace=i;
		}
	}else{
		//̃x
		/*
		 * AABB̑̐ςŏƂȂ镨߂AzA
		 * vZʂ̂ŁAxȕւȕ@ɂƂǂ߂
		 *
		 * ł́AAABB̒Sł߂mߎƂB
		 * ܂AA猩BœKȌł΁AƂB猩C(!=A)œKƂĂA
		 * A-B̃yAm肷
		 */

		std::set<DWORD> used;
		std::vector<DWORD> combines;	//̑g
		std::vector<DWORD> singles;		//łȂg
		for(DWORD idxA=0;idxA<numSubLevels;idxA++){
			//Ɍ肳Ă?
			if(!isIn(used,idxA)){
				//͖
				float fMinVolume=FLT_MAX;
				DWORD bestIdx=-1;
				for(DWORD idxB=idxA+1;idxB<numSubLevels;idxB++){
					float fVolume=combinationWeight(
						rTree[idxSubLevels+idxA].vMax,
						rTree[idxSubLevels+idxA].vMin,
						rTree[idxSubLevels+idxB].vMax,
						rTree[idxSubLevels+idxB].vMin);
					if(fVolume<fMinVolume){
						if(!isIn(used,idxB)){
							fMinVolume=fVolume;
							bestIdx=idxB;
						}
					}
				}
				if(bestIdx==-1){
					singles.push_back(idxA);
					used.insert(idxA);
				}else{
					combines.push_back(idxA*numSubLevels+bestIdx);
					used.insert(idxA);
					used.insert(bestIdx);
				}
			}
		}

		//
		rTree.resize(rTree.size()+combines.size()+singles.size());
		for(size_t nc=0;nc<combines.size();nc++){
			DWORD idxA=combines[nc]/numSubLevels;
			DWORD idxB=combines[nc]%numSubLevels;

			getCombinedAABB(
				rTree[dwOffset+nc].vMax,
				rTree[dwOffset+nc].vMin,
				rTree[idxSubLevels+idxA].vMax,
				rTree[idxSubLevels+idxA].vMin,
				rTree[idxSubLevels+idxB].vMax,
				rTree[idxSubLevels+idxB].vMin);

			rTree[dwOffset+nc].dwFace=-1;
			rTree[dwOffset+nc].children[0]=idxSubLevels+idxA;
			rTree[dwOffset+nc].children[1]=idxSubLevels+idxB;
		}
		//łȂAABB͂̂܂܊iグ
		for(size_t nc=0;nc<singles.size();nc++){
			rTree[dwOffset+combines.size()+nc]=rTree[idxSubLevels+singles[nc]];

			rTree[dwOffset+combines.size()+nc].dwFace=-1;
			rTree[dwOffset+combines.size()+nc].children[0]=idxSubLevels+singles[nc];
			rTree[dwOffset+combines.size()+nc].children[1]=-1;
		}
	}


	//ēx?
	if(rTree.size()-dwOffset>1){
		DXUtilBuildPolygonAABBTree(
			rTree,
			dwOffset,
			pVertices,
			vertexStride,
			numFaces,
			pIndexArray);
	}
}


//--------------------------------------------------------------------------------------
// Given a ray origin (orig) and direction (dir), and three vertices of a triangle, this
// function returns TRUE and the interpolated texture coordinates if the ray intersects 
// the triangle
//--------------------------------------------------------------------------------------
bool IntersectTriangle(	const MQPoint& orig, const MQPoint& dir, const MQPoint& v0, const MQPoint& v1, const MQPoint& v2, float* dist, float* u, float* v )
{
	// Find vectors for two edges sharing vert0
	MQPoint edge1 = v1 - v0;
	MQPoint edge2 = v2 - v0;

	// Begin calculating determinant - also used to calculate U parameter
	MQPoint pvec=GetExteriorProduct(dir, edge2 );

	// If determinant is near zero, ray lies in plane of triangle
	float det = GetInnerProduct( edge1, pvec );

	MQPoint tvec;
	if( det > 0.0f ){
		tvec = orig - v0;
	}else{
		tvec = v0 - orig;
		det = -det;
	}

	if( det < FLT_MIN ){
		return false;
	}

	// Calculate U parameter and test bounds
	*u = GetInnerProduct( tvec, pvec );
	if( *u < 0.0f || *u > det ){
		return false;
	}

	// Prepare to test V parameter
	MQPoint qvec=GetExteriorProduct( tvec, edge1 );

	// Calculate V parameter and test bounds
	*v = GetInnerProduct( dir, qvec );
	if( *v < 0.0f || *u + *v > det ){
		return false;
	}

	// Calculate t, scale parameters, ray intersects triangle
	*dist = GetInnerProduct( edge2, qvec );
	float fInvDet = 1.0f / det;
	*dist *= fInvDet;
	*u *= fInvDet;
	*v *= fInvDet;

	return true;
}

//L̒CƃbV̔(AABBp)
bool DXUtilIntersectLimitedRay(
	const MQPoint &vBegin,
	const MQPoint &vEnd,
	const std::vector<POLYGON_AABB_NODE> &rTree,
	const void* pVertices,
	size_t vertexStride,	//_z̈vf̑傫
	const UINT numIndices,
	const DWORD *pIndexArray,
	std::vector<RAYHITINFO> &hitPoints
)
{
	hitPoints.clear();
	ASSERT(!rTree.empty());
	if(rTree.empty())return false;

	MQPoint vDir=vEnd-vBegin;
	float fLenLim=vDir.abs();
	vDir.normalize();

	//ċAł͂Ȃ[vŎ邽߂̃X^bN
	std::stack<DWORD> local_stack;
	local_stack.push(rTree.size()-1);

	for(;!local_stack.empty();){
		DWORD dwIdx=local_stack.top();
		local_stack.pop();
		const POLYGON_AABB_NODE& rNode=rTree[dwIdx];

		//E`FbN
		if(DXUtilCheckBoxHit(rNode.vMax,rNode.vMin,vBegin,vEnd)){
			DWORD widx=rNode.dwFace;
			if(-1!=widx){
				UINT i0=pIndexArray[widx*3+0];
				UINT i1=pIndexArray[widx*3+1];
				UINT i2=pIndexArray[widx*3+2];

				const MQPoint &v0=*( (const MQPoint*) (((LPCBYTE)pVertices)+vertexStride*i0) );
				const MQPoint &v1=*( (const MQPoint*) (((LPCBYTE)pVertices)+vertexStride*i1) );
				const MQPoint &v2=*( (const MQPoint*) (((LPCBYTE)pVertices)+vertexStride*i2) );

				float fU,fV;
				float fDist;
				if(IntersectTriangle(vBegin,vDir,v0,v1,v2,&fDist,&fU,&fV)){
					RAYHITINFO rhi;
					rhi.fDist=fDist;
					rhi.fProd=GetInnerProduct(GetNormal(v0,v1,v2),vDir);
					hitPoints.push_back(rhi);
				}
			}

			//---qm[h
			for(int i=0;i<2;i++){
				if(-1!=rNode.children[i])local_stack.push(rNode.children[i]);
			}
		}
	}

	return !hitPoints.empty();
}


//LȒ_ȂfalseԂ
bool computeBoundaryBox(MQObject lpObject, MQPoint &vMax,MQPoint &vMin)
{
	size_t numVertices=lpObject->GetVertexCount();

	bool bFirst=true;
	for(size_t i=0;i<numVertices;i++){
		if(0!=lpObject->GetVertexRefCount(i)){
			MQPoint v=lpObject->GetVertex(i);
			if(bFirst){
				vMax=v;
				vMin=v;
				bFirst=false;
			}else{
				vMax.x=fmax(vMax.x,v.x);
				vMax.y=fmax(vMax.y,v.y);
				vMax.z=fmax(vMax.z,v.z);

				vMin.x=fmin(vMin.x,v.x);
				vMin.y=fmin(vMin.y,v.y);
				vMin.z=fmin(vMin.z,v.z);
			}
		}
	}
	return !bFirst;
}

void getObjectInfo(MQObject obj,std::vector<MQPoint> &vertices,std::vector<DWORD> &indices)
{
	//_z
	vertices.resize(obj->GetVertexCount());
	obj->GetVertexArray(&vertices[0]);

	//CfbNX
	int numFaces=obj->GetFaceCount();
	indices.clear();
	indices.reserve(numFaces*3);
	for(int fidx=0;fidx<numFaces;fidx++){
		int numPoints=obj->GetFacePointCount(fidx);
		if(numPoints>2){
			/*
			 * numPoints==0 :폜ς
			 * numPoints==2 :ӂ̂
			 *   ӂƂ͓蔻Ƃ邱ƂłȂ̂ŁAȂ
			 */
			int tmp[4];
			obj->GetFacePointArray(fidx,tmp);
			indices.push_back(tmp[0]);
			indices.push_back(tmp[1]);
			indices.push_back(tmp[2]);

			if(numPoints==4){
				indices.push_back(tmp[0]);
				indices.push_back(tmp[2]);
				indices.push_back(tmp[3]);
			}
		}
	}
}


//Cƃ|Sn_񋓂
void enumHitPointsAlongRay(std::vector<RAYHITINFO> &hitPoints,const std::vector<POLYGON_AABB_NODE> &rTree,const std::vector<MQPoint> &vertices,const std::vector<DWORD> &indices,MQPoint &vOrig,MQPoint &vDir,float lenMax)
{
	DXUtilIntersectLimitedRay(vOrig,vOrig+vDir*lenMax,rTree,&vertices[0],sizeof(vertices[0]),indices.size(),&indices[0],hitPoints);
	std::sort(hitPoints.begin(),hitPoints.end(),RAYHITINFO_SORTER());
}

bool isActiveVoxel(const std::vector<RAYHITINFO> &hitPoints,float fPos)
{
	//NOTE : 悭͂ÃASY͎኱GɂȂ
	//ł͒Pȕ@gɂ

	int flag=0;	//\tO
	for(unsigned int i=0;i<hitPoints.size();i++){
		if(hitPoints[i].fDist > fPos)break;
		if(hitPoints[i].fProd > 0.0f){
			flag++;
		}else{
			flag--;
		}
	}
	return flag%2!=0;
}


//iq\ʂ݂̂ɂ
/*
 * NOTE:
 * {NZiKASYς΂IɌvZ\B
 *   ->\ʂɑ΂Ă̂݃{NZ𐶐悤ȃASY
 * ł͊̃{NZ̉򂩂\ʂ̊iq݂̂𒊏oASYƂ
 */
void ExtractSurface(std::vector<VOXEL> &voxels,int nx,int ny,int nz)
{
	std::vector<BYTE> flags;
	flags.resize(nx*ny*nz);
	memset(&flags[0],0,sizeof(flags[0])*flags.size());

	for(unsigned int i=0;i<voxels.size();i++){
		int x=voxels[i].x;
		int y=voxels[i].y;
		int z=voxels[i].z;

		flags[z*(ny*nx)+y*nx+x]=1;
	}

	std::vector<VOXEL> retArray;
	retArray.reserve(voxels.size());

	//e{NZɂāA͂܂ĂȂXLbv
	for(unsigned int vi=0;vi<voxels.size();vi++){
		int x=voxels[vi].x;
		int y=voxels[vi].y;
		int z=voxels[vi].z;

		if(0<x && x<nx-1 && 0<y && y<ny-1 && 0<z && z<nz-1){
			for(int i=0;i<27;i++){
				//if(i==13)continue;
				int tx=i%3-1;
				int ty=(i/3)%3-1;
				int tz=i/9-1;
				if(tx*tx + ty*ty + tz*tz !=1) continue;	//΂ߕ̓XLȂ

				tx+=x; ty+=y; tz+=z;
				int idx=tz*(ny*nx)+ty*nx+tx;
				if(!flags[idx]){
					//͂܂ĂȂ
					retArray.push_back(voxels[vi]);
					break;
				}
			}
		}else{
			retArray.push_back(voxels[vi]);
		}
	}
	voxels=retArray;
}


#define SCAN_VOXEL(ax,ay,rayAxis)	\
{\
	for(int ay=0;ay<n##ay;ay++){\
		for(int ax=0;ax<n##ax;ax++){\
			MQPoint vOrg;\
			{\
				int rayAxis=0;\
				vOrg=MQPoint(x+0.5f,y+0.5f,z+0.5f)*fSize + vMin - ray*fSize;\
			}\
\
			/*ʂƃCꏊ𒲂ׂ*/\
			enumHitPointsAlongRay(hitPoints,tree,vertices,indices,vOrg,ray,vMax.##rayAxis-vMin.##rayAxis+1.0e-5f);\
\
			/*Ȃ疳*/\
			if(hitPoints.empty())continue;\
\
			/*eiqɂāAʂ̕\𔻒肷*/\
			for(int rayAxis=0;rayAxis<n##rayAxis;rayAxis++){\
				float pos=(rayAxis+1.0f)*fSize;	/*uC̔˓_vů݂iq̒Sv܂ł̋*/\
				if(isActiveVoxel(hitPoints,pos))voxels.push_back(VOXEL(x,y,z));\
			}\
		}\
	}\
}

bool divideVoxel(std::vector<VOXEL> &voxels,float fSize,MQObject lpObject,int axis,MQPoint &_vMin,BOOL bSurfaceOnly)
{
	MQPoint vMax,vMin;

	//E{bNXZo
	if(!computeBoundaryBox(lpObject,vMax,vMin))return false;

	_vMin=vMin;

	//Zo
	const int nx=int((vMax.x-vMin.x)/fSize)+1;
	const int ny=int((vMax.y-vMin.y)/fSize)+1;
	const int nz=int((vMax.z-vMin.z)/fSize)+1;

	voxels.reserve(nx*ny*nz);

	//CZo
	if(axis==-1){
		axis=0;
		if(nx<ny)axis=1;
		if(ny<nz)axis=2;
	}

	//
	std::vector<MQPoint> vertices;
	std::vector<DWORD> indices;
	getObjectInfo(lpObject,vertices,indices);

	//AABBc[\z:荂̂߂̃c[
	std::vector<POLYGON_AABB_NODE> tree;
	DXUtilBuildPolygonAABBTree(tree,0,&vertices[0],sizeof(vertices[0]),indices.size()/3,&indices[0]);

	//XL
	std::vector<RAYHITINFO> hitPoints;
	MQPoint ray;
	switch(axis){
	case 0:	//X
		ray=MQPoint(1.0f, 0.0f, 0.0f);
		SCAN_VOXEL(y,z,x);
		break;
	case 1:	//Y
		ray=MQPoint(0.0f, 1.0f, 0.0f);
		SCAN_VOXEL(z,x,y);
		break;
	case 2:	//Z
		ray=MQPoint(0.0f, 0.0f, 1.0f);
		SCAN_VOXEL(x,y,z);
		break;
	default:
		return false;
	}

	if(bSurfaceOnly){
		ExtractSurface(voxels,nx,ny,nz);
	}

	return true;
}



/*
//{NZ𐶐
void SpawnVoxels(MQObject lpVoxelized,const std::vector<VOXEL>& voxels,float fSize,MQPoint &vMin)
{
	MQObject tmpObj=MQ_CreateObject();
	//{NZ:CIuWFNgɃ}[W
	//TODO:Œ_ʒu͓Kł悢
	int vertices[]={
		tmpObj->AddVertex(MQPoint(0.0f,0.0f,0.0f)),	//0
		tmpObj->AddVertex(MQPoint(0.0f,0.0f,0.0f)),	//1
		tmpObj->AddVertex(MQPoint(0.0f,0.0f,0.0f)),	//2
		tmpObj->AddVertex(MQPoint(0.0f,0.0f,0.0f)),	//3
		tmpObj->AddVertex(MQPoint(0.0f,0.0f,0.0f)),	//4
		tmpObj->AddVertex(MQPoint(0.0f,0.0f,0.0f)),	//5
		tmpObj->AddVertex(MQPoint(0.0f,0.0f,0.0f)),	//6
		tmpObj->AddVertex(MQPoint(0.0f,0.0f,0.0f)),	//7
	};

	int indices[]={
		4,5,1,0,//0,1,5,4,
		1,2,3,0,//0,3,2,1,
		3,7,4,0,//0,4,7,3,
		7,6,5,4,//4,5,6,7,
		6,7,3,2,//2,3,7,6,
		5,6,2,1,//1,2,6,5,
	};

	for(int j=0;j<COUNTOF(indices);j++){
		indices[j]=vertices[indices[j]];
	}

	for(int j=0;j<6;j++){
		// ʂǉ
		tmpObj->AddFace(4, &indices[j*4]);
	}

	//}[W
	for(unsigned int i=0;i<voxels.size();i++){
		float x=voxels[i].x * fSize+vMin.x;
		float y=voxels[i].y * fSize+vMin.y;
		float z=voxels[i].z * fSize+vMin.z;

		float s=fSize;

		//_XV
		tmpObj->SetVertex(vertices[0],MQPoint(x  ,y  ,z  )),	//0
		tmpObj->SetVertex(vertices[1],MQPoint(x+s,y  ,z  )),	//1
		tmpObj->SetVertex(vertices[2],MQPoint(x+s,y+s,z  )),	//2
		tmpObj->SetVertex(vertices[3],MQPoint(x  ,y+s,z  )),	//3
		tmpObj->SetVertex(vertices[4],MQPoint(x  ,y  ,z+s)),	//4
		tmpObj->SetVertex(vertices[5],MQPoint(x+s,y  ,z+s)),	//5
		tmpObj->SetVertex(vertices[6],MQPoint(x+s,y+s,z+s)),	//6
		tmpObj->SetVertex(vertices[7],MQPoint(x  ,y+s,z+s)),	//7

		lpVoxelized->Merge(tmpObj);
	}
	//lpVoxelized->Compact();
	tmpObj->DeleteThis();
}
*/
//{NZ𐶐
void SpawnVoxels(MQObject lpVoxelized,const std::vector<VOXEL>& voxels,float fSize,MQPoint &vMin)
{
	for(unsigned int i=0;i<voxels.size();i++){
		float x=voxels[i].x * fSize + vMin.x;
		float y=voxels[i].y * fSize + vMin.y;
		float z=voxels[i].z * fSize + vMin.z;

		float s=fSize;

		int vertices[]={
			lpVoxelized->AddVertex(MQPoint(x  ,y  ,z  )),	//0
			lpVoxelized->AddVertex(MQPoint(x+s,y  ,z  )),	//1
			lpVoxelized->AddVertex(MQPoint(x+s,y+s,z  )),	//2
			lpVoxelized->AddVertex(MQPoint(x  ,y+s,z  )),	//3
			lpVoxelized->AddVertex(MQPoint(x  ,y  ,z+s)),	//4
			lpVoxelized->AddVertex(MQPoint(x+s,y  ,z+s)),	//5
			lpVoxelized->AddVertex(MQPoint(x+s,y+s,z+s)),	//6
			lpVoxelized->AddVertex(MQPoint(x  ,y+s,z+s)),	//7
		};

		int indices[]={
			4,5,1,0,//0,1,5,4,
			1,2,3,0,//0,3,2,1,
			3,7,4,0,//0,4,7,3,
			7,6,5,4,//4,5,6,7,
			6,7,3,2,//2,3,7,6,
			5,6,2,1,//1,2,6,5,
		};

		for(int j=0;j<COUNTOF(indices);j++){
			indices[j]=vertices[indices[j]];
		}

		for(int j=0;j<6;j++){
			// ʂǉ
			lpVoxelized->AddFace(4, &indices[j*4]);
		}
	}
}


//_̂ݐ;pr
void SpawnVertices(MQObject lpVoxelized,const std::vector<VOXEL>& voxels,float fSize,MQPoint &vMin)
{
	for(unsigned int i=0;i<voxels.size();i++){
		float x=voxels[i].x * fSize + vMin.x + fSize*0.5f;
		float y=voxels[i].y * fSize + vMin.y + fSize*0.5f;
		float z=voxels[i].z * fSize + vMin.z + fSize*0.5f;

		int idx=lpVoxelized->AddVertex(MQPoint(x  ,y  ,z  ));

		lpVoxelized->AddFace(1,&idx);
	}
}

